local jutils = require("utility.jutils")
local items = require("items")
local tiles = require("tiles")
local rendering = require("utility.rendering")
local recipes = require("recipes")
local inventory = require("inventory.inventory")
local playerinventory = inventory:subclass("PlayerInventory")
local grid = require("utility.grid")

local openPos = {0, 1, 2, -180}
local closedPos = {0, 1, 2, -40}

function playerinventory:init()
	inventory.init(self, jutils.point.new(10, 4), closedPos, 32)
	self.fullyshown = false
	self.hotbarslot = 1
	self.recipeLookingAt = nil
	self.player = nil
end

function playerinventory:fullOpen()
	self.screenpos = openPos
	self.uiresponsive = true
	self.fullyshown = true
	self.pointingat = -1
end

function playerinventory:fullClose()
	self.screenpos = closedPos
	self.uiresponsive = false
	self.fullyshown = false
end

function playerinventory:getHeldItem()

	if self.itemDragging[1] > 0 then
		return self.itemDragging
	end
	return self.items[self.hotbarslot]
end

function playerinventory:clicked(x, y, button, touch, number)
	

	if self.recipeLookingAt ~= nil then

		for _, reactant in pairs(self.recipeLookingAt.reactants) do
			self:removeItem(items[reactant[1]].id, reactant[2])
		end

		for _, product in pairs(self.recipeLookingAt.products) do
			self:addItem(items[product[1]].id, product[2])
		end
		return true
	end

	return inventory.clicked(self, x, y, button, touch, number)
end

function playerinventory:update(dt)
	inventory.update(self, dt)

	if self.fullyshown then
		self:input()
	else
		--self.pointingat = self.hotbarslot
	end
end

local rarityColors = {
	[-1] = {0, 0, 0},
	[0] = {0.6, 0.6, 0.6},
	[1] = {1,1,1},
	[2] = {0.5, 1, 0.5},
	[3] = {0.5, 0.5, 1},
	[4] = {1, 0.5, 0.5},
	[5] = {1, 0.5, 1},
	[6] = {1, 1, 0.5},
}

function playerinventory:drawhotbar()
	local screenx, screeny = self:getDrawPosition()

	local item = self.items[self.hotbarslot]
	if item[1] > 0 then
		local iteminfo = items:getByID(item[1])
		love.graphics.setColor(rarityColors[iteminfo.rarity])
		rendering:setFont("pixelade_normal")
		love.graphics.print(iteminfo.displayname, screenx+2, screeny-20)
		rendering:setFont("default")
	end
end

function playerinventory:draw()
	love.graphics.push()
	love.graphics.origin()

	rendering:setFont("pixelade_normal")
	for idx, effect in pairs(self.player.statuseffects) do
		love.graphics.setColor(0, 0, 0, 0.5)

		local text = effect.id..": "..math.floor(effect.time).."s"

		local f=love.graphics.getFont()
		local len = f:getWidth(text)

		love.graphics.rectangle("fill", love.graphics.getWidth()-(len+15), (idx)*25 + 5, len+5, 24, 5, 5)

		love.graphics.setColor(1,1,1)

	
		love.graphics.printf(text, love.graphics.getWidth()-(len+15), idx*25+5, len+5, "center")
	end

	self.recipeLookingAt = nil

	if self.fullyshown then
		love.graphics.setColor(0, 0, 0, 0.3)
		love.graphics.rectangle("fill", 0, 0, love.graphics.getWidth(), love.graphics.getHeight())

		local mx, my = love.mouse.getPosition()

		local slotY = 0

		local startx = 32
		local starty = 128

		
		for _, recipe in pairs(recipes) do
			local hasRecipe = true


			local range = 10

			local playerworld = self.player.world
			local playerx, playery = grid.pixelToTileXY(self.player.position.x, self.player.position.y)
			if recipe.station ~= nil then
				local nearby = false
				for x = -range, range do
					for y = -range, range do
						local tile = playerworld:getTile(playerx+x, playery+y)

						if jutils.table.contains(tiles:getByID(tile).tags, recipe.station) then
							nearby = true
						end

						if recipe.station == nil then nearby = true end
					end
				end

				if nearby == false then hasRecipe = false end
			end

			for _, reactant in pairs(recipe.reactants) do
				if not self:hasItem(items[reactant[1]].id, reactant[2]) then
					hasRecipe = false
				end
			end

			if hasRecipe then
				

				local num = #recipe.reactants + #recipe.products + 1
				local boxX = startx-4
				local boxY = starty+(slotY*32)-4
				local boxW = (num*32)
				local boxH = 32

				love.graphics.setColor(1,1,1, 0.5)
				if mx > boxX and mx < boxX+boxW and my > boxY and my < boxY+boxH then
					love.graphics.setColor(1,1,1)
					self.recipeLookingAt = recipe
				end
				
				love.graphics.rectangle("fill", boxX, boxY, boxW, boxH)
				love.graphics.setColor(1,1,1)
				local slotX = 0
				for _, reactant in pairs(recipe.reactants) do
					local iteminfo = items[reactant[1]]
					
					rendering:drawItem(iteminfo.id, startx+(slotX*32), starty+(slotY*32), 3, 0)
				
					if reactant[2] > 1 then
						love.graphics.setColor(0,0,0)
						
						love.graphics.printf(reactant[2], startx+(slotX*32), starty+(slotY*32)+15, 32, "right")
						
					end
					slotX = slotX + 1
				end

				love.graphics.printf("->", startx+(slotX*32), starty+ (slotY*32), 32, "center")
				slotX = slotX + 1
				for _, product in pairs(recipe.products) do
					local iteminfo = items[product[1]]
					rendering:drawItem(iteminfo.id, startx+(slotX*32), starty+(slotY*32), 3, 0)

					if product[2] > 1 then
						love.graphics.setColor(0,0,0)
						love.graphics.printf(product[2], startx+(slotX*32), starty+(slotY*32)+15, 32, "right")
					end
					slotX = slotX + 1
				end

				slotY = slotY + 1
			end
		end
		rendering:setFont("default")



		for index = 1, #self.items do
			if index == self.pointingat then
				self:drawSlotHighlighted(index, {1,1,1})
			elseif index == self.hotbarslot then
				self:drawSlotHighlighted(index, {1,1,1, 0.25})
			else
				self:drawSlot(index)
			end
		end
		self:drawpointingdata()
	else
		for index = 1, 10 do
			if index == self.hotbarslot then
				self:drawSlotHighlighted(index, {1,1,1})
			else
				self:drawSlot(index)
			end
		end
		self:drawhotbar()
	end

	self:drawDragging()
	love.graphics.pop()
end

return playerinventory