--[[
	cavegame project

	team:
		joshuu - programming, design, art, testing

	contributors:
		bumpy - testing, art
		BillyJ - testing
		Mescalyne - inspiring this project & teaching me many skills

	technologies used:
		GNU Image Manipulation Program
		Love2D
		Lua Programming Language
		Itch.io hosting
		Visual Studio Code & it's many wonderful plugins
]]

local config = require("config")
local jutils = require("utility.jutils")
local gameloop = require("gameloop")

local fullscreen = false

-- game setup and other bullshit
function love.load(cmdlineargs)

	love.window.setTitle("cavegame 0.3 alpha")
	love.window.setMode(
		1000, 600,
		{
			fullscreen = false,
			resizable = true,
			minwidth = 500,
			minheight = 300,
			vsync = false
		}
	)

	gameloop.initialize()
	gameloop.resize(1000, 600)
end

function love.quit()

end

function love.keypressed(key)
	gameloop.keypressed(key)

	if key == "f11" then
		fullscreen = not fullscreen

		love.window.setFullscreen(fullscreen)
	end
end

function love.keyreleased(...) gameloop.keyreleased(...) end
function love.mousepressed(...) gameloop.mousepressed(...) end
function love.mousereleased(...) gameloop.mousereleased(...) end
function love.wheelmoved(...) gameloop.wheelmoved(...) end
function love.textinput(...) gameloop.textinput(...) end
function love.update(dt) gameloop.update(dt) end
function love.resize(width, height) gameloop.resize(width, height) end

-- stats display, I don't really know why I made this
local statvalue = {}
local statlabel = {}
local statssuffix = {}

local idx = 1

local function addStat(label, value, suffix)
	statvalue[idx] = value
	statlabel[idx] = label
	statssuffix[idx] = (suffix ~= nil) and suffix or ""
	idx = idx + 1
end

local function drawstats()
	local text = ""

	for index, label in ipairs(statlabel) do
		local value = statvalue[index]
		local suffix = statssuffix[index]

		local addition = ""
		if type(value) == "table" then
			for idx, inst in ipairs(value) do
				if idx < #value then
					addition = addition .. inst .. ","
				else
					addition = addition .. inst
				end
			end
		else
			addition = tostring(value)
		end
		text = text .. label .. ": " .. addition .. suffix .. " "
	end

	love.graphics.setColor(0, 0, 0)
	love.graphics.print(text, 2, 2)
end

function love.draw()
	
	gameloop.draw()
	
	local stats = love.graphics.getStats()
	local luavmmem = collectgarbage("count")

	if config.debug.SHOW_DATA then
		addStat("fps", love.timer.getFPS())
		addStat("dt", jutils.math.round(love.timer.getAverageDelta() * 1000, 3), "ms")
		addStat("luavm-memory", jutils.math.round(luavmmem / 1000, 1), "mb")

		addStat("\nimg", stats.images)
		addStat("txmem", jutils.math.round(stats.texturememory / 1000, 1), "kb")
		addStat("dc", stats.drawcalls)
		addStat("batched-dc", stats.drawcallsbatched)

		local gamedata = gameloop.info()

		addStat("\nmousepos", gamedata.mousepos)
		addStat("id", gamedata.tileid)
		addStat("name", gamedata.tilename)
		addStat("state", gamedata.state)
		addStat("light", gamedata.light)
		addStat("damage", gamedata.damage)

		drawstats()

		statssuffix = {}
		statlabel = {}
		statvalue = {}
		idx = 1
	else
		love.graphics.setColor(1, 1, 1)
		love.graphics.print("fps: " .. love.timer.getFPS(), 2, 2)
	end

	love.graphics.setColor(0, 0, 0, 0.5)

end
